<?php

namespace Arma\Internal;

use Arma\IMat;
use Arma\Complex;

/**
 * Interface for objects with a dense layout (Mat, Col and Row).
 *
 * @package Arma\Internal
 */
interface Dense extends Base
{
    /**
     * Element-wise equality evaluation of two objects.
     *
     * @param static $other
     * @return IMat
     */
    function equals($other);

    /**
     * Element-wise non-equality evaluation of two objects.
     *
     * @param static $other
     * @return IMat
     */
    function notEquals($other);

    /**
     * Element-wise "greater than" evaluation of two objects.
     *
     * @param static $other
     * @return IMat
     */
    function greaterThan($other);

    /**
     * Element-wise "smaller than" evaluation of two objects.
     *
     * @param static $other
     * @return IMat
     */
    function smallerThan($other);

    /**
     * Element-wise "not greater than" evaluation of two objects.
     *
     * @param static $other
     * @return IMat
     */
    function notGreaterThan($other);

    /**
     * Element-wise "not smaller than" evaluation of two objects.
     *
     * @param static $other
     * @return IMat
     */
    function notSmallerThan($other);

    /**
     * Sets the elements to a specified value. The type of value must match the type of elements used by
     * the container object.
     *
     * For matrices, filling is done column-by-column.
     *
     * @param number|Complex $value
     * @return void
     */
    function fill($value);

    /**
     * Fill with values provided by a callback function.
     *
     * For matrices, filling is done column-by-column.
     *
     * @param callable $callback () => $fill_value
     * @return void
     */
    function imbue($callback);
}
