<?php

namespace Arma\Internal;

/**
 * Interface for resizable (non-subview) vectors.
 *
 * @package Arma\Internal
 */
interface ResizableVector extends Resizable, Vector
{
    /**
     * Recreate the object according to given size specifications, while preserving the elements
     * as well as the layout of the elements.
     *
     * @param int $n_elem
     * @return void
     */
    function resize($n_elem);

    /**
     * Change the size of an object, without explicitly preserving data and without initialising the elements.
     *
     * @param int $n_elem
     * @return void
     */
    function setSize($n_elem);

    /**
     * Set the elements of an object to zero, optionally first changing the size to specified dimensions.
     *
     * @param int $n_elem[optional]
     * @return $this
     */
    function zeros($n_elem);

    // Characteristics

    /**
     * Check whether the given index is currently valid.
     *
     * @param int $idx
     * @return bool
     */
    function inRange($idx);
}
