--TEST--
Test save/load with HDF5 format for `Arma\Mat`.
--SKIPIF--
<?php
require_once 'includes/loaded.php';
require_once 'includes/supports.php';
if (is_php_arma_loaded()) {
    supports_hdf5();
}
?>
--ENV--
TMP_FILE=/tmp/php_arma_test.tmp
--FILE--
<?php

require_once 'includes/assert.php';

$file_name = getenv('TMP_FILE');

$hdf5_options = [
    'file_name' => $file_name,
    'dataset_name' => 'tmp_dataset',
    'options' => Arma\Hdf5Opts::TRANS
];

$mat = Arma\IMat::init(3, 3, Arma\Fill::RANDN);
batch_assert('saving with HDF5 format',
    [true, $mat->save($hdf5_options, Arma\FileType::HDF5_BINARY)]
);

$mat1 = Arma\IMat::init();
batch_assert('loading with HDF5 format',
    [true, $mat1->load($hdf5_options, Arma\FileType::HDF5_BINARY)]
);

$eq_mat = $mat->equals($mat1);
batch_assert('loading with HDF5 format',
    [1, $eq_mat->min()],
    [1, $eq_mat->max()]
);

?>
--CLEAN--
<?php
$file_name = getenv('TMP_FILE');
if (file_exists($file_name)) {
    unlink($file_name);
}
?>
--EXPECT--
